if (typeof _497BFC326A484FAC8AB6B8CD0D886127 === "undefined") {
	_497BFC326A484FAC8AB6B8CD0D886127 = {};
}

(function(){
var global = this;
if (global.common === undefined) {
	global.common = {};
}
if (global.log === undefined) {
	global.log = {};
}
if (global.login === undefined) {
	global.login = {};
}
if (global.experimental === undefined) {
	global.experimental = {};
}

/***************************/
/* START utility functions */
/**
 * Copy all the prototype from parent to child but doesn't overwrite anything.
 */
global.common.objectExtend = function(child, parent) {
	if (!child || !parent) {
		throw "child or parent is not defined";
	}
	for (var p in parent.prototype ) {
		if (!child.prototype[p]) {
			child.prototype[p] = parent.prototype[p];
		}
	}
};

global.log.LOG_LEVEL = {
	OFF:   99999,
	ERROR: 1000,
	WARN:  900,
	INFO:  800,
	DEBUG: 500,
	TRACE: 300,
	ON:    0
};
global.log.logPrefix = global.brand.logPrefix;

/* END utility functions */
/*************************/



global.common.DefaultValues = {
	debugMode: false,
	debugDelay: 0
};


/************************/
/* START CentrifyBEBase */
/**
 * The base class of everything
 */
global.common.CentrifyBEBase = function() {};
global.common.CentrifyBEBase.prototype = {
	/**
	 * If debugMode is true, there will be a delay between the transition .
	 * And logging lower than error will show up.
	 */
	debugMode: global.common.DefaultValues.debugMode,
	
	loggerName: "",
	
	logLevel: global.log.LOG_LEVEL.INFO,
	
	getTime: function() {
		var date = new Date();
		var h = date.getHours();
		h = h < 10 ? "0" + h : h;
		var m = date.getMinutes();
		m = m < 10 ? "0" + m : m;
		var s = date.getSeconds();
		s = s < 10 ? "0" + s : s;
		var ms = date.getMilliseconds();
		ms = ms < 100 ? ms < 10 ? "00" + ms : "0" + ms : ms;
		return h + ":" + m + ":" + s + "." + ms;
	},
	
	/**
	 * Returns the first element within the document (using depth-first 
	 *  pre-order traversal of the document's nodes) that matches the 
	 *  specified group of selectors.
	 * @param {Document} doc
	 * @param {string{ selectors A string containing one or more CSS selectors 
	 *        separated by commas
	 * @return {Element} The matched element object.
	 */
	querySelector: function(doc, selectors) {
		try {
			return doc.querySelector(selectors);
		} catch (err) {
			this.log("querySelector: selectors = " + selectors);
			throw err;
		}
	},
	
	/**
	 * check the current document url's protocol against the loginData's url protocol.
	 * @param {docUrl}
	 * @param {loginDataUrl} loginData.applicationUrl
	 * @return {boolean} return true if it is safe to login
	 */
	checkProtocol: function(docUrl, loginDataUrl) {
		if (loginDataUrl && docUrl) {
			// IE 11 doesn't support URL object, and loginDataUrl is a string, so use regular expression
			var protocolReg = new RegExp(/^[^:]*:/);
			// extract the urls' protocol
			var curUrlProtocolMatch = docUrl.match(protocolReg)
			var loginUrlProtocolMatch = loginDataUrl.match(protocolReg)
			if (curUrlProtocolMatch && loginUrlProtocolMatch) {
				curUrlProtocol = curUrlProtocolMatch[0];
				loginUrlProtocol = loginUrlProtocolMatch[0];
				if (loginUrlProtocol === "https:" && curUrlProtocol === "http:") {
					// user configured as https, but the actually login page is http
					this.log("LoginData protocol is https, but current url protocol http, unsecure situatoin, can't login");
					return false;
				} else {
					// bypass other situations
					this.log("LoginData protocol=" + loginUrlProtocol + " and current url protocol=" +
						curUrlProtocol + ", proceed to login");
					return true;
				}
			}
			else{
				this.log("can't find protocols in document url: " + docUrl + 
					" or applicationUrl: " + loginDataUrl + ", for security reason, skip login");
			}
		}
		this.log("invalid urls: document url=" + docUrl + ", applicationUrl=" + loginDataUrl);
		return false
	},
	
	/**
	 * The default way to log a raw message. Should not be called directly.
	 * @param {string} message
	 */
	toLogger: function(message) {
		console.log(message);
	},
	
	/**
	 * Is the level currently enabled?
	 * For level lower or equals to INFO, the debug mode must be on.
	 * @param {global.log.LOG_LEVEL} level
	 * @return {boolean}
	 */
	isLogLevelEnabled: function(level) {
		if (this.logLevel <= level) {
			if (this.debugMode) {
				return true;
			} else {
				//don't log anything less than info if debugMode is not on.
				return level > global.log.LOG_LEVEL.INFO;
			}
		}
		return false;
	},
	
	/**
	 * return a displayable level string
	 * @param {global.log.LOG_LEVEL} level
	 * @return {string}
	 */
	getLevelString: function(level) {
		if (level >= global.log.LOG_LEVEL.OFF)   return "NOTIC";
		if (level >= global.log.LOG_LEVEL.ERROR) return "ERROR";
		if (level >= global.log.LOG_LEVEL.WARN)  return "WARN ";
		if (level >= global.log.LOG_LEVEL.INFO)  return "INFO ";
		if (level >= global.log.LOG_LEVEL.DEBUG) return "DEBUG";
		return "TRACE";
	},
	
	/**
	 * log the message with log level.
	 * @param {global.log.LOG_LEVEL} level
	 * @param {string} message
	 */
	logWithLevel: function(level, message) {
		if (this.isLogLevelEnabled(level)) {
			this.logWithEverything(global.log.logPrefix, this.getTime(), this.getLevelString(level), this.loggerName, message);
		}
	},
	
	/**
	 * @param {string} prefix
	 * @param {string} time
	 * @param {string} level
	 * @param {string} loggerName
	 * @param {string} message
	 */
	logWithEverything: function(prefix, time, level, loggerName, message) {
		this.toLogger(prefix + " " + time + " " + level + " [" + loggerName +"]: " + message);
	},
	
	/**
	 * @param {string} message
	 */
	logNotice: function(message) {
		this.logWithLevel(global.log.LOG_LEVEL.OFF, message);
	},
	
	/**
	 * @param {string} message
	 */
	logTrace: function(message) {
		this.logWithLevel(global.log.LOG_LEVEL.TRACE, message);
	},
	
	/**
	 * @param {string} message
	 */
	logDebug: function(message) {
		this.logWithLevel(global.log.LOG_LEVEL.DEBUG, message);
	},
	
	/**
	 * @param {string} message
	 */
	log: function(message) {
		this.logWithLevel(global.log.LOG_LEVEL.INFO, message);
	},
	
	/**
	 * @param {string} message
	 */
	logWarn: function(message) {
		this.logWithLevel(global.log.LOG_LEVEL.WARN, message);
	},
	
	/**
	 * @param {string} message
	 */
	logError: function(message) {
		this.logWithLevel(global.log.LOG_LEVEL.ERROR, message);
	},
	
	/**
	 * @param {string} loggingStr
	 * @return {global.log.LOG_LEVEL}
	 */
	getMyLogLevel: function(loggingStr) {
		var level;
		try {
			if (loggingStr) {
				var loggingObj = JSON.parse(loggingStr);
				var newLevelStr = loggingObj[this.loggerName];
				if (newLevelStr) {
					level = global.log.LOG_LEVEL[newLevelStr];
				}
			}
		} catch(e) {
			this.logWarn("incorrect logging level " + e);
		}
		if (!level) {
			level = global.log.LOG_LEVEL.INFO;
		}
		return level;
	}
	
	/****************************/
	/* START abstract functions */
	
	/* END abstract functions */
	/**************************/
};
/* END CentrifyBEBase */
/**********************/





global.login.SchemeEnum = {
	httpHttps: "HttpHttps", // http and https
	httpOnly: "HttpOnly", // http only
	https: "Https", // https only
	ftp: "Ftp",
	basic: "Basic",
	digest: "Digest",
	any: "Any"
};

global.login.LoginOrderEnum = {
	/**
	 * @since 1.103
	 */
	click: "click",
	/**
	 * @since 1.103
	 * changed on 1.113
	 */
	newPage: "waitForNewPage",
	/**
	 * username/password/submit
	 * @since 1.103
	 */
	ups: "ups",
	/**
	 * @since 1.104
	 */
	sleep: "sleep",
	/**
	 * @since 1.106
	 */
	fill: "fill",
	/**
	 * @since 1.106
	 */
	submit: "submit",
	/**
	 * @since 1.106
	 */
	gotoUrl: "goto",
	/**
	 * @since 1.107
	 */
	fillEnter: "fillEnter",
	/**
	 * @since 1.113
	 */
	header401: "header401",
	/**
	 * @since 1.140
	 */
	iframe: "iframe"
};
global.login.Common = {
	/**
	 */
	initDefaultLoginOrder: function(loginData) {
		switch (loginData.scheme) {
			case global.login.SchemeEnum.httpHttps:
			case global.login.SchemeEnum.httpOnly:
			case global.login.SchemeEnum.https:
			case global.login.SchemeEnum.any:
				loginData.order = [[global.login.LoginOrderEnum.ups]];
				break;
			case global.login.SchemeEnum.ftp:
			case global.login.SchemeEnum.basic:
			case global.login.SchemeEnum.digest:
				loginData.order = [[global.login.LoginOrderEnum.header401]];
				break;
			default:
				// why here? null? Anyway, safest approach
				loginData.order = [[global.login.LoginOrderEnum.ups]];
				break;
		}
	}
};





global.experimental.EventEnum = {
	login: "cbelogin"
};
global.experimental.consoleLoginListener = function(aEvent) {
	var fromDoc = aEvent.target 
			? (
				aEvent.target.ownerDocument 
					? aEvent.target.ownerDocument 
					: aEvent.target 
			)
			: document;
	
	var data = global.experimental.consoleLoginListener.getCbeLoginData(fromDoc);
	if (!data || !data.value) {
		window.alert("you forget to set cbeLogin");
		return;
	}
	var loginData = JSON.parse(data.value);
	global.login.centrifyBELogin.setLoginData(fromDoc, loginData);
	global.experimental.consoleLoginListener.redirect(fromDoc, loginData);
};

global.experimental.consoleLoginListener.redirect = function(doc, loginData) {
	global.login.centrifyBELogin.log("redirecting to " + loginData.applicationUrl);
	doc.location = loginData.applicationUrl;
};

global.experimental.consoleLoginListener.getCbeLoginData = function(doc) {
	return doc.getElementById("cbeLogin");
};

if (global.experimental.parameter === undefined) {
	global.experimental.parameter = {};
}

global.experimental.parameter.KnownParameters = {
	enableConsoleLogin: "enable-console-login"
};

global.experimental.parameter.isEnable = function(parameters, key) {
	if (parameters && key) {
		return  parameters.toLowerCase().indexOf(key) != -1;
	} else {
		return false;
	}
};

}).call(_497BFC326A484FAC8AB6B8CD0D886127);
